
/*

	Packet capture example program with pcap library

Taeho Oh ( ohhara@4dl.com, ohhara@postech.edu )   http://postech.edu/~ohhara
4DL ( 4th Dimension Laboratory )                              http://4dl.com
Postech ( Pohang University of Science and Technology )   http://postech.edu

*/

#include<stdio.h>
#include<stdlib.h>
#include<netinet/ether.h>
#include<netinet/ip.h>
#include<netinet/tcp.h>
#include<netinet/udp.h>
#include<netinet/ip_icmp.h>
#include<pcap.h>

char errbuf[PCAP_ERRBUF_SIZE];

u_char *packet;
struct ether_header *ether;
struct iphdr *ip;
struct tcphdr *tcp;
struct udphdr *udp;
struct icmphdr *icmp;
u_char *data;

const int etherhdrsize=sizeof(struct ether_header);
const int iphdrsize=sizeof(struct iphdr);
const int tcphdrsize=sizeof(struct tcphdr);
const int udphdrsize=sizeof(struct udphdr);
const int icmphdrsize=sizeof(struct icmphdr);

void dump_packet()
{
	int i;
	int len;
	unsigned char *ptr=packet;

	len=etherhdrsize;

	printf("------------------------------------------------\n");

	printf("ether->ether_dhost     :   %02x:%02x:%02x:%02x:%02x:%02x\n",ether->ether_dhost[0],ether->ether_dhost[1],ether->ether_dhost[2],ether->ether_dhost[3],ether->ether_dhost[4],ether->ether_dhost[5]);
	printf("ether->ether_shost     :   %02x:%02x:%02x:%02x:%02x:%02x\n",ether->ether_shost[0],ether->ether_shost[1],ether->ether_shost[2],ether->ether_shost[3],ether->ether_shost[4],ether->ether_shost[5]);
	printf("ether->ether_type      : %10u %8x\n",ntohs(ether->ether_type),ntohs(ether->ether_type));

	if(ntohs(ether->ether_type)==ETHERTYPE_IP)
	{
		len+=ntohs(ip->tot_len);
		printf("ip->version            : %10u %8x\n",ip->version,ip->version);
		printf("ip->ihl                : %10u %8x\n",ip->ihl,ip->ihl);
		printf("ip->tos                : %10u %8x\n",ip->tos,ip->tos);
		printf("ip->tot_len            : %10u %8x\n",ntohs(ip->tot_len),ntohs(ip->tot_len));
		printf("ip->id                 : %10u %8x\n",ntohs(ip->id),ntohs(ip->id));
		printf("ip->frag_off           : %10u %8x\n",ntohs(ip->frag_off),ntohs(ip->frag_off));
		printf("ip->ttl                : %10u %8x\n",ip->ttl,ip->ttl);
		printf("ip->protocol           : %10u %8x\n",ip->protocol,ip->protocol);
		printf("ip->check              : %10u %8x\n",ntohs(ip->check),ntohs(ip->check));
		printf("ip->saddr              : %10u %8x\n",ntohl(ip->saddr),ntohl(ip->saddr));
		printf("ip->saddr              : %19s\n",(char *)inet_ntoa(ip->saddr));
		printf("ip->daddr              : %10u %8x\n",ntohl(ip->daddr),ntohl(ip->daddr));
		printf("ip->daddr              : %19s\n",(char *)inet_ntoa(ip->daddr));
/*		printf("ip->saddr              : %10u %8x\n",ip->saddr,ip->saddr);
		printf("ip->saddr              : %19s\n",(char *)inet_ntoa(ip->saddr));
		printf("ip->daddr              : %10u %8x\n",ip->daddr,ip->daddr);
		printf("ip->daddr              : %19s\n",(char *)inet_ntoa(ip->daddr));*/

		if(ip->protocol==IPPROTO_TCP)
		{
			printf("tcp->source            : %10u %8x\n",ntohs(tcp->source),ntohs(tcp->source));
			printf("tcp->dest              : %10u %8x\n",ntohs(tcp->dest),ntohs(tcp->dest));
			printf("tcp->seq               : %10u %8x\n",ntohl(tcp->seq),ntohl(tcp->seq));
			printf("tcp->ack_seq           : %10u %8x\n",ntohl(tcp->ack_seq),ntohl(tcp->ack_seq));
			printf("tcp->doff              : %10u %8x\n",tcp->doff,tcp->doff);
			printf("tcp->res1              : %10u %8x\n",tcp->res1,tcp->res1);
			printf("tcp->res2              : %10u %8x\n",tcp->res2,tcp->res2);
			printf("tcp->urg               : %10u %8x\n",tcp->urg,tcp->urg);
			printf("tcp->ack               : %10u %8x\n",tcp->ack,tcp->ack);
			printf("tcp->psh               : %10u %8x\n",tcp->psh,tcp->psh);
			printf("tcp->rst               : %10u %8x\n",tcp->rst,tcp->rst);
			printf("tcp->syn               : %10u %8x\n",tcp->syn,tcp->syn);
			printf("tcp->fin               : %10u %8x\n",tcp->fin,tcp->fin);
			printf("tcp->window            : %10u %8x\n",ntohs(tcp->window),ntohs(tcp->window));
			printf("tcp->check             : %10u %8x\n",ntohs(tcp->check),ntohs(tcp->check));
			printf("tcp->urg_ptr           : %10u %8x\n",ntohs(tcp->urg_ptr),ntohs(tcp->urg_ptr));
		}
		else if(ip->protocol==IPPROTO_UDP)
		{
			printf("udp->source            : %10u %8x\n",ntohs(udp->source),ntohs(udp->source));
			printf("udp->dest              : %10u %8x\n",ntohs(udp->dest),ntohs(udp->dest));
			printf("udp->len               : %10u %8x\n",ntohs(udp->len),ntohs(udp->len));
			printf("udp->check             : %10u %8x\n",ntohs(udp->check),ntohs(udp->check));
		}
		else if(ip->protocol==IPPROTO_ICMP)
		{
			printf("icmp->type             : %10u %8x\n",icmp->type,icmp->type);
			printf("icmp->code             : %10u %8x\n",icmp->code,icmp->code);
			printf("icmp->checksum         : %10u %8x\n",ntohs(icmp->checksum),ntohs(icmp->checksum));
			printf("icmp->un.echo.id       : %10u %8x\n",ntohs(icmp->un.echo.id),ntohs(icmp->un.echo.id));
			printf("icmp->un.echo.sequence : %10u %8x\n",ntohs(icmp->un.echo.sequence),ntohs(icmp->un.echo.sequence));
			printf("icmp->un.gateway       : %10u %8x\n",ntohl(icmp->un.gateway),ntohl(icmp->un.gateway));
			printf("icmp->un.frag.__unused : %10u %8x\n",ntohs(icmp->un.frag.__unused),ntohs(icmp->un.frag.__unused));
			printf("icmp->un.frag.mtu      : %10u %8x\n",ntohs(icmp->un.frag.mtu),ntohs(icmp->un.frag.mtu));
		}
		else
		{
			printf("supported transport layer protocol: tcp, udp, icmp\n");
		}
	}
	else
	{
		printf("supported network layer protocol: ip\n");
	}

	for(i=0;i<len;i++)
	{
		if(!(i&15))
			printf("\n");
		printf(" ");
		printf("%02x",ptr[i]);
	}
	printf("\n");

	for(i=0;i<len;i++)
	{
		if(!(i&15))
			printf("\n");
		if(isprint(ptr[i]))
			printf("  %c",ptr[i]);
		else
			printf("  .");
	}
	printf("\n");
	printf("------------------------------------------------\n");
}

main(int argc,char **argv)
{
	pcap_t *cap;
	char *dev;
	struct pcap_pkthdr pkthdr;

	dev=pcap_lookupdev(errbuf);
	if(dev==NULL)
	{
		printf("%s\n",errbuf);
		exit(-1);
	}

	cap=pcap_open_live(dev,0,0,0,errbuf);
	if(cap==NULL)
	{
		printf("%s\n",errbuf);
		exit(-1);
	}

	if(pcap_datalink(cap)!=DLT_EN10MB)
	{
		printf("supported datalink layer protocol: ethernet\n");
		exit(-1);
	}

	while(1)
	{
		packet=(u_char *)pcap_next(cap,&pkthdr);
		ether=(struct ether_header *)packet;
		ip=(struct iphdr *)(packet+etherhdrsize);
		udp=(struct udphdr *)(packet+etherhdrsize+iphdrsize);
		tcp=(struct tcphdr *)(packet+etherhdrsize+iphdrsize);
		icmp=(struct icmphdr *)(packet+etherhdrsize+iphdrsize);
		dump_packet();
		fflush(stdout);
	}
	pcap_close(cap);
}
